package ru.dbotthepony.mc.otm.datagen.advancements

import net.minecraft.advancements.Advancement
import net.minecraft.advancements.RequirementsStrategy
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraftforge.common.data.ExistingFileHelper
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.core.registryName
import ru.dbotthepony.mc.otm.datagen.lang.MatteryLanguageProvider
import ru.dbotthepony.mc.otm.datagen.modLocation
import ru.dbotthepony.mc.otm.registry.MItemTags
import ru.dbotthepony.mc.otm.registry.MItems
import java.util.function.Consumer

private data class CraftEntry(val item: Item, val englishName: String, val englishSuffix: String? = null)

fun addMachineAdvancements(serializer: Consumer<Advancement>, existingFileHelper: ExistingFileHelper, lang: MatteryLanguageProvider) {
	val translation = lang.english.Prepended("otm.advancements.machine")

	val root = AdvancementBuilder()
		.requirements(RequirementsStrategy.OR)
		.display(
			itemStack = ItemStack(MItems.CHEMICAL_GENERATOR),
			title = TranslatableComponent(translation.add("root", "Tritanium Empowered Machinery")),
			description = TranslatableComponent(translation.add("root.desc", "Do not drop in anything if you want the latter preserved intact")),
			showToast = false,
			announceChat = false,
			background = modLocation("textures/block/decorative/floor_tiles_gray.png")
		)
		.addCriterion("has_machine", criterion(MItemTags.MACHINES))
		.addCriterion("has_tritanium_ingot", criterion(MItemTags.TRITANIUM_INGOTS))
		.addCriterion("has_tritanium_plate_somehow", criterion(MItemTags.TRITANIUM_PLATES))
		.save(serializer, modLocation("machines/root"), existingFileHelper)

	val chem = AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(MItems.CHEMICAL_GENERATOR),
			title = TranslatableComponent(translation.add("chemical_generator", "Burning the Organics")),
			description = TranslatableComponent(translation.add("chemical_generator.desc", "Craft a Chemical Generator, better to put it outside")),
		)
		.addCriterion("has_machine", criterion(MItems.CHEMICAL_GENERATOR))
		.save(serializer, modLocation("machines/chemical_generator"), existingFileHelper)

	val press = AdvancementBuilder()
		.parent(chem)
		.display(
			itemStack = ItemStack(MItems.PLATE_PRESS),
			title = TranslatableComponent(translation.add("plate_press", "Bending the Material")),
			description = TranslatableComponent(translation.add("plate_press.desc", "Craft a Plate Press, avoid having limbs inside during operation")),
		)
		.addCriterion("has_machine", criterion(MItems.PLATE_PRESS))
		.save(serializer, modLocation("machines/plate_press"), existingFileHelper)

	val entries = listOf(
		CraftEntry(MItems.MATTER_SCANNER, "Scanning Things that Matter"),
		CraftEntry(MItems.PATTERN_STORAGE, "Digital Knowledge Library"),
		CraftEntry(MItems.MATTER_DECOMPOSER, "Decaying the Atoms", "Keep your limbs outside of the working chamber at all times"),
		CraftEntry(MItems.MATTER_PANEL, "Indexing the Library"),
		CraftEntry(MItems.MATTER_REPLICATOR, "Local Bakery", "Now let's bake some perfect bread"),
		CraftEntry(MItems.MATTER_BOTTLER, "Transfusing Pure Matter", "For those who loved to play with water in their childhood"),
		CraftEntry(MItems.MATTER_RECYCLER, "Refine and Redefine", "This is what waste recycling should look like"),
		CraftEntry(MItems.MATTER_CAPACITOR_BANK, "Modular Matter Tank"),

		CraftEntry(MItems.ENERGY_COUNTER, "Visualize Power Burn"),
		CraftEntry(MItems.BATTERY_BANK, "Batteries Not Included", "By all means avoid the urge to hammer incompatible batteries into the power bus."),
	)

	val built = mutableMapOf<Item, Advancement>()

	for (entry in entries) {
		val path = entry.item.registryName!!.path

		built[entry.item] = AdvancementBuilder()
			.parent(press)
			.display(
				itemStack = ItemStack(entry.item),
				title = TranslatableComponent(translation.add(path, entry.englishName)),
				description = TranslatableComponent(translation.add("$path.desc", "Craft a %s%s"), entry.item.description, if (entry.englishSuffix != null) ". " + entry.englishSuffix else ""),
			)
			.addCriterion("has_machine", criterion(entry.item))
			.save(serializer, modLocation("machines/$path"), existingFileHelper)
	}
}
