package ru.dbotthepony.mc.otm.datagen.recipes

import net.minecraft.data.recipes.*
import net.minecraft.resources.ResourceLocation
import net.minecraft.tags.ItemTags
import net.minecraft.world.item.DyeColor
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Items
import net.minecraft.world.item.crafting.Ingredient
import net.minecraft.world.level.ItemLike
import net.minecraftforge.common.Tags
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.core.registryName
import ru.dbotthepony.mc.otm.datagen.DataGen
import ru.dbotthepony.mc.otm.registry.MBlocks
import ru.dbotthepony.mc.otm.registry.MItemTags
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MRegistry
import java.util.function.Consumer

private fun stairs(base: ItemLike, result: ItemLike, consumer: Consumer<FinishedRecipe>) {
	MatteryRecipe(result, 4)
		.rowA(base)
		.rowAB(base, base)
		.row(base, base, base)
		.unlockedBy(base)
		.build(consumer)
}

private fun slab(base: ItemLike, result: ItemLike, consumer: Consumer<FinishedRecipe>) {
	MatteryRecipe(result, 6)
		.row(base, base, base)
		.unlockedBy(base)
		.build(consumer)
}

private fun wall(base: ItemLike, result: ItemLike, consumer: Consumer<FinishedRecipe>) {
	MatteryRecipe(result, 6)
		.row(base, base, base)
		.row(base, base, base)
		.unlockedBy(base)
		.build(consumer)
}

private fun cut(base: ItemLike, result: ItemLike, amount: Int, consumer: Consumer<FinishedRecipe>) {
	SingleItemRecipeBuilder
		.stonecutting(Ingredient.of(base), result, amount)
		.unlockedBy(base)
		.save(consumer, ResourceLocation(OverdriveThatMatters.MOD_ID, "stonecutting/${result.asItem().registryName!!.path}_from_${base.asItem().registryName!!.path}"))
}

private fun stairsWithCut(base: ItemLike, result: ItemLike, consumer: Consumer<FinishedRecipe>) {
	stairs(base, result, consumer)
	cut(base, result, 1, consumer)
}

private fun slabWithCut(base: ItemLike, result: ItemLike, consumer: Consumer<FinishedRecipe>) {
	slab(base, result, consumer)
	cut(base, result, 2, consumer)
}

private fun wallWithCut(base: ItemLike, result: ItemLike, consumer: Consumer<FinishedRecipe>) {
	wall(base, result, consumer)
	cut(base, result, 1, consumer)
}

fun addDecorativesRecipes(provider: MatteryRecipeProvider, consumer: Consumer<FinishedRecipe>) {
	// Напольная плитка
	for ((color, unrefinedItem) in MRegistry.UNREFINED_FLOOR_TILES.items) {
		MatteryRecipe(unrefinedItem, 24)
			.row(Items.CLAY, Items.SAND, Items.CLAY)
			.row(Items.SAND, color.tag, Items.SAND)
			.row(Items.CLAY, Items.SAND, Items.CLAY)
			.unlockedBy(Items.CLAY)
			.build(consumer)
	}

	for ((color, refinedItem) in MRegistry.FLOOR_TILES.items) {
		SimpleCookingRecipeBuilder.smelting(
			Ingredient.of(MRegistry.UNREFINED_FLOOR_TILES.items[color]!!),
			refinedItem,
			0.15f,
			200
		).unlockedBy("has_unrefined_block", has(MRegistry.UNREFINED_FLOOR_TILES.items[color]!!)).save(consumer)
	}

	for ((color, item) in MRegistry.TRITANIUM_BLOCK.allItems) {
		stairsWithCut(item, MRegistry.TRITANIUM_STAIRS.allItems[color]!!, consumer)
		slabWithCut(item, MRegistry.TRITANIUM_SLAB.allItems[color]!!, consumer)
		wallWithCut(item, MRegistry.TRITANIUM_WALL.allItems[color]!!, consumer)
	}

	for ((item, colors) in MRegistry.TRITANIUM_STRIPED_BLOCK.itemsWithColor) {
		val (a, b) = colors
		stairsWithCut(item, MRegistry.TRITANIUM_STRIPED_STAIRS.getItem(a, b), consumer)
		slabWithCut(item, MRegistry.TRITANIUM_STRIPED_SLAB.getItem(a, b), consumer)
		wallWithCut(item, MRegistry.TRITANIUM_STRIPED_WALL.getItem(a, b), consumer)
	}

	stairsWithCut(MItems.TRITANIUM_STRIPED_BLOCK, MItems.TRITANIUM_STRIPED_STAIRS, consumer)
	slabWithCut(MItems.TRITANIUM_STRIPED_BLOCK, MItems.TRITANIUM_STRIPED_SLAB, consumer)
	wallWithCut(MItems.TRITANIUM_STRIPED_BLOCK, MItems.TRITANIUM_STRIPED_WALL, consumer)

	for ((color, item) in MRegistry.FLOOR_TILES.items) {
		stairsWithCut(item, MRegistry.FLOOR_TILES_STAIRS.items[color]!!, consumer)
		slabWithCut(item, MRegistry.FLOOR_TILES_SLAB.items[color]!!, consumer)
	}

	for ((item, colors) in MRegistry.TRITANIUM_STRIPED_BLOCK.itemsWithColor) {
		val (base, stripe) = colors

		val original = MRegistry.TRITANIUM_BLOCK.items[base]!!

		MatteryRecipe(item, 8)
			.row(original, original, original)
			.row(original, stripe.tag, original)
			.row(original, original, original)
			.unlockedBy(original)
			.build(consumer)
	}

	MatteryRecipe(MItems.DANGER_STRIPE_BLOCK, 24)
		.rowAB(Tags.Items.DYES_YELLOW, MItemTags.IRON_PLATES)
		.row(MItemTags.IRON_PLATES, Tags.Items.COBBLESTONE, MItemTags.IRON_PLATES)
		.rowBC(MItemTags.IRON_PLATES, Tags.Items.DYES_BLACK)
		.unlockedBy(MItemTags.IRON_PLATES)
		.unlockedBy(Tags.Items.DYES_BLACK)
		.build(consumer)

	MatteryRecipe(MItems.METAL_BEAM, 24)
		.rowB(MItemTags.IRON_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, Tags.Items.COBBLESTONE, MItemTags.TRITANIUM_PLATES)
		.rowB(MItemTags.IRON_PLATES)
		.build(consumer)

	MatteryRecipe(MRegistry.VENT.item, 24)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, Items.IRON_BARS, MItemTags.TRITANIUM_PLATES)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	for ((color, item) in MRegistry.VENT.items) {
		MatteryRecipe(item, 8)
			.row(MRegistry.VENT.item, MRegistry.VENT.item, MRegistry.VENT.item)
			.row(MRegistry.VENT.item, color.tag, MRegistry.VENT.item)
			.row(MRegistry.VENT.item, MRegistry.VENT.item, MRegistry.VENT.item)
			.unlockedBy(MItemTags.TRITANIUM_PLATES)
			.unlockedBy(color.tag)
			.build(consumer)
	}

	for ((color, item) in MRegistry.VENT_ALTERNATIVE.allItems) {
		val other = MRegistry.VENT.allItems[color]!!
		ShapelessRecipeBuilder(item, 1).requires(other).unlockedBy(item).save(consumer)
		ShapelessRecipeBuilder(other, 1).requires(item).unlockedBy(other).save(consumer, ResourceLocation(
			OverdriveThatMatters.MOD_ID, "${other.registryName!!.path}_from_alt"))
	}

	MatteryRecipe(MRegistry.DECORATIVE_CRATE.item, 24)
		.rowB(MItemTags.IRON_PLATES)
		.row(MItemTags.IRON_PLATES, Tags.Items.COBBLESTONE, MItemTags.IRON_PLATES)
		.rowB(MItemTags.IRON_PLATES)
		.unlockedBy(MItemTags.IRON_PLATES)
		.build(consumer)

	for ((color, crate) in MRegistry.DECORATIVE_CRATE.items) {
		MatteryRecipe(crate, 8)
			.row(MRegistry.DECORATIVE_CRATE.item, MRegistry.DECORATIVE_CRATE.item, MRegistry.DECORATIVE_CRATE.item)
			.row(MRegistry.DECORATIVE_CRATE.item, color.tag, MRegistry.DECORATIVE_CRATE.item)
			.row(MRegistry.DECORATIVE_CRATE.item, MRegistry.DECORATIVE_CRATE.item, MRegistry.DECORATIVE_CRATE.item)
			.unlockedBy(MRegistry.DECORATIVE_CRATE.item)
			.build(consumer)
	}

	for ((color, item) in MRegistry.TRITANIUM_BLOCK.items) {
		MatteryRecipe(item, 8)
			.row(MRegistry.TRITANIUM_BLOCK.item, MRegistry.TRITANIUM_BLOCK.item, MRegistry.TRITANIUM_BLOCK.item)
			.row(MRegistry.TRITANIUM_BLOCK.item, color.tag, MRegistry.TRITANIUM_BLOCK.item)
			.row(MRegistry.TRITANIUM_BLOCK.item, MRegistry.TRITANIUM_BLOCK.item, MRegistry.TRITANIUM_BLOCK.item)
			.unlockedBy(MRegistry.TRITANIUM_BLOCK.item)
			.unlockedBy(color.tag)
			.build(consumer)
	}

	ShapelessRecipeBuilder(MRegistry.INDUSTRIAL_GLASS.item, 8)
		.requires(Items.GLASS, 8)
		.requires(MItemTags.TRITANIUM_PLATES)
		.unlockedBy("has_plate", has(MItemTags.TRITANIUM_PLATES))
		.unlockedBy("has_glass", has(Items.GLASS))
		.save(consumer)

	ShapedRecipeBuilder(MRegistry.INDUSTRIAL_GLASS_PANE.item, 16)
		.define('#', MRegistry.INDUSTRIAL_GLASS.item)
		.pattern("###").pattern("###")
		.unlockedBy("has_tritanium_glass", has(MRegistry.INDUSTRIAL_GLASS.item))
		.save(consumer)

	val mappingUpgradeVanilla = mapOf(
		DyeColor.WHITE to Items.WHITE_STAINED_GLASS,
		DyeColor.ORANGE to Items.ORANGE_STAINED_GLASS,
		DyeColor.MAGENTA to Items.MAGENTA_STAINED_GLASS,
		DyeColor.LIGHT_BLUE to Items.LIGHT_BLUE_STAINED_GLASS,
		DyeColor.YELLOW to Items.YELLOW_STAINED_GLASS,
		DyeColor.LIME to Items.LIME_STAINED_GLASS,
		DyeColor.PINK to Items.PINK_STAINED_GLASS,
		DyeColor.GRAY to Items.GRAY_STAINED_GLASS,
		DyeColor.LIGHT_GRAY to Items.LIGHT_GRAY_STAINED_GLASS,
		DyeColor.CYAN to Items.CYAN_STAINED_GLASS,
		DyeColor.PURPLE to Items.PURPLE_STAINED_GLASS,
		DyeColor.BLUE to Items.BLUE_STAINED_GLASS,
		DyeColor.BROWN to Items.BROWN_STAINED_GLASS,
		DyeColor.GREEN to Items.GREEN_STAINED_GLASS,
		DyeColor.RED to Items.RED_STAINED_GLASS,
		DyeColor.BLACK to Items.BLACK_STAINED_GLASS,
	)

	for (color in DyeColor.values()) {
		val item = MRegistry.INDUSTRIAL_GLASS.items[color]!!
		val paneItem = MRegistry.INDUSTRIAL_GLASS_PANE.items[color]!!
		val mappedVanilla = mappingUpgradeVanilla[color]!!

		// обычная покраска
		ShapelessRecipeBuilder(item, 8)
			.requires(MRegistry.INDUSTRIAL_GLASS.item, 8)
			.requires(color.tag)
			.unlockedBy("has_tritanium_glass", has(MRegistry.INDUSTRIAL_GLASS.item))
			.save(consumer)

		// апгрейд ванильного крашенного стекла
		ShapelessRecipeBuilder(item, 8)
			.requires(mappedVanilla, 8)
			.requires(MItemTags.TRITANIUM_PLATES)
			.unlockedBy("has_plate", has(MItemTags.TRITANIUM_PLATES))
			.unlockedBy("has_colored_glass", has(mappedVanilla))
			.save(consumer, ResourceLocation(DataGen.MOD_ID, "${item.registryName!!.path}_upgrade"))

		ShapelessRecipeBuilder(paneItem, 8)
			.requires(MRegistry.INDUSTRIAL_GLASS_PANE.item, 8)
			.requires(color.tag)
			.unlockedBy("has_tritanium_glass_pane", has(MRegistry.INDUSTRIAL_GLASS_PANE.item))
			.save(consumer)

		ShapedRecipeBuilder(paneItem, 16)
			.define('#', item)
			.pattern("###").pattern("###")
			.unlockedBy("has_colored_tritanium_glass", has(paneItem))
			.save(consumer, ResourceLocation(DataGen.MOD_ID, "${paneItem.registryName!!.path}_alt"))
	}

	ShapelessRecipeBuilder(MItems.LABORATORY_LAMP, 1)
		.requires(MItems.LABORATORY_LAMP_INVERTED)
		.unlockedBy(MItems.LABORATORY_LAMP_INVERTED)
		.save(consumer, MItems.LABORATORY_LAMP.registryName!!.toString() + "_inv")

	ShapelessRecipeBuilder(MItems.LABORATORY_LAMP_INVERTED, 1)
		.requires(MItems.LABORATORY_LAMP)
		.unlockedBy(MItems.LABORATORY_LAMP)
		.save(consumer, MItems.LABORATORY_LAMP_INVERTED.registryName!!.toString() + "_inv")

	MatteryRecipe(MBlocks.TRITANIUM_STRIPED_BLOCK, 24)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, Tags.Items.COBBLESTONE, MItemTags.TRITANIUM_PLATES)
		.rowAB(Tags.Items.DYES_YELLOW, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.unlockedBy(Tags.Items.DYES_YELLOW)
		.build(consumer)

	MatteryRecipe(MRegistry.TRITANIUM_BLOCK.item, 24)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, Tags.Items.COBBLESTONE, MItemTags.TRITANIUM_PLATES)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MBlocks.TRITANIUM_DOOR, 3)
		.rowAB(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.rowAB(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.rowAB(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MBlocks.TRITANIUM_TRAPDOOR)
		.rowAB(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.rowAB(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MRegistry.TRITANIUM_PRESSURE_PLATE.item)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.BASIC_CIRCUIT, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	for (dye in DyeColor.values()) {
		ShapelessRecipeBuilder(MRegistry.TRITANIUM_PRESSURE_PLATE.getItem(dye), 1)
			.requires(Ingredient.of(MRegistry.TRITANIUM_PRESSURE_PLATE.allItems.entries.stream().filter { it.key != dye }.map { ItemStack(it.value) }))
			.requires(dye.tag)
			.unlockedBy(MItemTags.TRITANIUM_PLATES)
			.save(consumer)
	}
}
