package ru.dbotthepony.mc.otm.block.entity.matter

import net.minecraft.MethodsReturnNonnullByDefault
import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.Container
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.block.BatteryBankBlock
import ru.dbotthepony.mc.otm.block.IDroppableContainer
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.matter.IMatterHandler
import ru.dbotthepony.mc.otm.capability.matter.MatterDirection
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.graph.Graph6Node
import ru.dbotthepony.mc.otm.graph.matter.IMatterGraphNode
import ru.dbotthepony.mc.otm.graph.matter.MatterNetworkGraph
import ru.dbotthepony.mc.otm.menu.MatterCapacitorBankMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.container.set
import ru.dbotthepony.mc.otm.core.map
import ru.dbotthepony.mc.otm.core.set
import javax.annotation.ParametersAreNonnullByDefault

class MatterCapacitorBankBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) : MatteryBlockEntity(MBlockEntities.MATTER_CAPACITOR_BANK, p_155229_, p_155230_), IMatterGraphNode, IMatterHandler, IDroppableContainer {
	var gaugeLevel by synchronizer.float()
		private set

	override val matterNode = Graph6Node<IMatterGraphNode>(this)
	private val resolverNode = LazyOptional.of { this }

	override val storedMatter: ImpreciseFraction get() {
		var summ = ImpreciseFraction.ZERO

		for (stack in container)
			if (!stack.isEmpty)
				stack.getCapability(MatteryCapability.MATTER).ifPresent {
					summ += it.storedMatter
				}

		return summ
	}

	override val maxStoredMatter: ImpreciseFraction get() {
		var summ = ImpreciseFraction.ZERO

		for (stack in container)
			if (!stack.isEmpty)
				stack.getCapability(MatteryCapability.MATTER).ifPresent {
					summ += it.maxStoredMatter
				}

		return summ
	}

	override fun receiveMatterOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		return receiveMatterInner(howMuch, simulate)
	}

	override fun receiveMatterInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		if (!howMuch.isPositive)
			return ImpreciseFraction.ZERO

		@Suppress("NAME_SHADOWING")
		var howMuch = howMuch
		var summ = ImpreciseFraction.ZERO

		for (stack in container) {
			if (!stack.isEmpty) {
				stack.getCapability(MatteryCapability.MATTER).ifPresent {
					val diff = it.receiveMatterOuter(howMuch, simulate)
					summ += diff
					howMuch -= diff
				}

				if (howMuch.isZero) {
					break
				}
			}
		}

		if (summ.isPositive && !simulate) {
			gaugeLevel = (storedMatter / maxStoredMatter).toFloat()
		}

		return summ
	}

	override fun extractMatterOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		return extractMatterInner(howMuch, simulate)
	}

	override fun extractMatterInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		if (!howMuch.isPositive)
			return ImpreciseFraction.ZERO

		@Suppress("NAME_SHADOWING")
		var howMuch = howMuch
		var summ = ImpreciseFraction.ZERO

		for (stack in container) {
			if (!stack.isEmpty) {
				stack.getCapability(MatteryCapability.MATTER).ifPresent {
					val diff = it.extractMatterOuter(howMuch, simulate)
					summ += diff
					howMuch -= diff
				}

				if (howMuch.isZero) {
					break
				}
			}
		}

		if (summ.isPositive && !simulate) {
			gaugeLevel = (storedMatter / maxStoredMatter).toFloat()
		}

		return summ
	}

	override val direction: MatterDirection get() {
		return MatterDirection.BIDIRECTIONAL
	}

	private var resolver = LazyOptional.of { this }

	val container = object : MatteryContainer(this::setChangedLight, 6 * 2) {
		override fun setChanged(slot: Int, new: ItemStack, old: ItemStack) {
			super.setChanged(slot, new, old)
			val level = level

			if (level != null) {
				var state = blockState

				for (i in BatteryBankBlock.BATTERY_SLOTS_PROPS.indices) {
					state = state.setValue(
						BatteryBankBlock.BATTERY_SLOTS_PROPS[i],
						getItem(i).getCapability(MatteryCapability.MATTER).isPresent
					)
				}

				if (state !== blockState) {
					level.setBlock(blockPos, state, Block.UPDATE_CLIENTS)
				}
			}

			gaugeLevel = (storedMatter / maxStoredMatter).toFloat()
		}
	}

	override val droppableContainer: Container
		get() = container
	override val defaultDisplayName: Component
		get() = MACHINE_NAME

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt[INVENTORY_KEY] = container.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)
		nbt.map(INVENTORY_KEY, container::deserializeNBT)
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return MatterCapacitorBankMenu(containerID, inventory, this)
	}

	private var valid = true

	override fun invalidateCaps() {
		super.invalidateCaps()
		valid = false
		resolver.invalidate()
	}

	override fun reviveCaps() {
		super.reviveCaps()
		valid = true
		resolver = LazyOptional.of { this }
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (valid) {
			if (cap === MatteryCapability.MATTER) return resolver.cast()
			if (cap === MatteryCapability.MATTER_NODE) return resolverNode.cast()
		}

		return super.getCapability(cap, side)
	}

	override fun setRemoved() {
		super.setRemoved()
		matterNode.destroy(::MatterNetworkGraph)
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel)
			MatterNetworkGraph.discoverFull(this, matterNode)
	}

	override fun getMatterHandler(): IMatterHandler {
		return this
	}

	companion object {
		private val MACHINE_NAME = TranslatableComponent("block.overdrive_that_matters.matter_capacitor_bank")
	}
}
