package ru.dbotthepony.mc.otm.menu

import com.google.common.collect.ImmutableList
import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.inventory.Slot
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.block.entity.EnergyServoBlockEntity
import ru.dbotthepony.mc.otm.capability.energy
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import ru.dbotthepony.mc.otm.registry.MMenus

class EnergyServoMenu @JvmOverloads constructor(
	p_38852_: Int,
	inventory: Inventory,
	tile: EnergyServoBlockEntity? = null
) : MatteryMenu(MMenus.ENERGY_SERVO, p_38852_, inventory, tile) {
	val dischargeSlot: MatterySlot
	val chargeSlot: MatterySlot

	val powerGauge = LevelGaugeWidget(this, tile?.energy)

	init {
		val container = tile?.container ?: SimpleContainer(2)

		dischargeSlot = object : MatterySlot(container, EnergyServoBlockEntity.SLOT_DISCHARGE) {
			override fun mayPlace(p_40231_: ItemStack): Boolean {
				return super.mayPlace(p_40231_) && (p_40231_.energy?.let { it.extractEnergy(Int.MAX_VALUE, true) > 0 } ?: false)
			}
		}

		chargeSlot = object : MatterySlot(container, EnergyServoBlockEntity.SLOT_CHARGE) {
			override fun mayPlace(p_40231_: ItemStack): Boolean {
				return super.mayPlace(p_40231_) && (p_40231_.energy?.let { it.receiveEnergy(Int.MAX_VALUE, true) > 0 } ?: false)
			}
		}

		addSlot(dischargeSlot)
		addSlot(chargeSlot)
		addInventorySlots()
	}

	override val storageSlots: Collection<Slot> = ImmutableList.of(dischargeSlot, chargeSlot)
}
