package ru.dbotthepony.mc.otm.menu

import net.minecraft.world.Container
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.Slot
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.energy
import ru.dbotthepony.mc.otm.capability.matter.MatterDirection
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.runOnClient

open class MatterySlot @JvmOverloads constructor(container: Container, index: Int, x: Int = 0, y: Int = 0) : Slot(container, index, x, y) {
	var ignoreSpectators = true

	override fun mayPickup(player: Player): Boolean {
		return super.mayPickup(player) && (!ignoreSpectators || !player.isSpectator)
	}

	override fun mayPlace(itemStack: ItemStack): Boolean {
		return super.mayPlace(itemStack) && (!ignoreSpectators || runOnClient(true) { minecraft.player?.isSpectator != true })
	}

	open fun canTakeItemForPickAll(): Boolean {
		return true
	}
}

open class MachineOutputSlot @JvmOverloads constructor(container: Container, index: Int, x: Int = 0, y: Int = 0) : MatterySlot(container, index, x, y) {
	override fun mayPlace(itemStack: ItemStack): Boolean {
		return false
	}
}

open class BatterySlot @JvmOverloads constructor(container: Container, index: Int, x: Int = 0, y: Int = 0) : MatterySlot(container, index, x, y) {
	override fun mayPlace(itemStack: ItemStack): Boolean {
		return super.mayPlace(itemStack) && (itemStack.energy?.canExtract() ?: false)
	}
}

open class MatterContainerInputSlot @JvmOverloads constructor(
	container: Container,
	index: Int,
	x: Int = 0,
	y: Int = 0,
	val direction: MatterDirection = MatterDirection.BIDIRECTIONAL
) : MatterySlot(container, index, x, y) {
	override fun mayPlace(itemStack: ItemStack): Boolean {
		val handler = itemStack.getCapability(MatteryCapability.MATTER).resolve()
		if (handler.isEmpty) return false
		val direction = handler.get().direction
		return super.mayPlace(itemStack) && (direction == MatterDirection.BIDIRECTIONAL || this.direction == direction)
	}
}

open class PatternSlot(container: Container, index: Int, x: Int = 0, y: Int = 0) : MatterySlot(container, index, x, y) {
	override fun mayPlace(itemStack: ItemStack): Boolean {
		return super.mayPlace(itemStack) && itemStack.getCapability(MatteryCapability.PATTERN).isPresent
	}
}

open class DriveSlot(container: Container, index: Int, x: Int = 0, y: Int = 0) : MatterySlot(container, index, x, y) {
	override fun mayPlace(itemStack: ItemStack): Boolean {
		return super.mayPlace(itemStack) && itemStack.getCapability(MatteryCapability.DRIVE).isPresent
	}
}
